//******************************************************************************

//******************************************************************************

/* .......................................................................................
 .......................................................................................*/

#include  "device_430.h"
#include  "integer.h"
#include  "HE_util.h"
#include  "sd.h"
#include "spi_sd.h"
#include "spi.h"

#include "HEDisk.h"

HEDskVolInfo heVol;
HEDskFileInfo heFileInfo;
BYTE sectorBuffer[ 512 ];
BYTE* secBuf= sectorBuffer;

//static unsigned char sd_buffer[512];
static sd_context_t sdc;

int main(void)
{
	/* Stop the watchdog timer */
	WDTCTL = WDTPW | WDTHOLD;

	P1DIR= BIT4; //testing......

	/* Set some reasonable values for the timeouts. */
	sdc.timeout_write = PERIPH_CLOCKRATE/8;
	sdc.timeout_read = PERIPH_CLOCKRATE/20;
	sdc.busyflag = 0;

	int ok= 0;
	int j;
	for( j= 0; j < SD_INIT_TRY && ok != 1; j++ )
	{
		ok= do_sd_initialize( &sdc );
	}

	HEDsk_LoadVolumeInfo( );
	
	if( HEDsk_GetFileInfo( &heFileInfo, "TEST    TXT" ) )
	{
		for( j= 0; j < 100; ++j )
		{
			P1OUT ^= BIT4; // Toggle P1.1 using exclusive-OR
			HEDsk_WriteFile( &heFileInfo, "this is a test\r\n", 16 );
		}
		HEDsk_FlushFile( &heFileInfo );
	}
	//pseudo flush
	GetSectorData( 0 );

	//testing
	while( 1 )
		;
}

bool GetSectorDataSPI( DWORD sector )
{
	if( sd_read_block( &sdc, sector, secBuf ) )
	{
		sd_wait_notbusy ( &sdc );
		return true;
	}
	return false;
}

bool SetSectorDataSPI( DWORD sector )
{
	if( sd_write_block( &sdc, sector, secBuf ) )
	{
		sd_wait_notbusy ( &sdc );
		return true;
	}
	return false;
}

bool acstrcmp( const char* a, const char* b, unsigned int count )
{
	for( ; count--; )
	{
		if( *a != *b )
			return false;
		++a;
		++b;
	}
	return true;
}

void HEDsk_memcpy( BYTE* pDest, BYTE* pSource, WORD count )
{
	for( ; count--; )
	{
		*pDest++= *pSource++;
	}
}

bool GetSectorData( DWORD sector )
{

	//cached
	if( heVol.curSector == sector )
		return true;

	if( heVol.volFlags & HEDS_VF_DIRTY
		&& ! SetSectorDataSPI( heVol.curSector )
	)
		return false;

	heVol.curSector= sector;
	//clear dirty
	heVol.volFlags&= ~HEDS_VF_DIRTY;

	return GetSectorDataSPI( sector );
}

bool SetSectorData( DWORD sector )
{

	if( heVol.curSector == sector )
	{
		//just mark it
		heVol.volFlags|= HEDS_VF_DIRTY;
		return true;
	}
	//else a new sector, was the last one dirty?
	if( heVol.volFlags & HEDS_VF_DIRTY )
	{
		if( ! SetSectorDataSPI( heVol.curSector ) )
			return false;
	}
	heVol.curSector= sector;

	//Note that the compiler will put this and the
	// other 'just mark it' together with a jump
	heVol.volFlags|= HEDS_VF_DIRTY;
	return true;
}



